@@ -1,6 +1,4 @@
 import autoarray as aa
-import os
-
 import numpy as np
 import pytest
 
@@ -1837,128 +1835,264 @@ class TestSubGrid2DFromSubGrid1d:
         ).all()
 
 
-class TestPositionsAtCoordinate:
-    def test__uniform_grid__locates_pixels_correctly(self):
+class TestGridPixelCentres1dViaOverlay:
+    def test__overlays_grid_using_pixel_scale(self):
 
-        grid = aa.Grid.uniform(shape_2d=(5, 5), pixel_scales=1.0)
-
-        pixels_at_coordinate = aa.util.grid.positions_at_coordinate_from(
-            grid_2d=grid.in_2d, coordinate=(0.3, 0.3)
-        )
-
-        assert pixels_at_coordinate == [(1, 2), (1, 3), (2, 2), (2, 3)]
-
-        pixels_at_coordinate = aa.util.grid.positions_at_coordinate_from(
-            grid_2d=grid.in_2d, coordinate=(-0.3, 0.3)
-        )
-
-        assert pixels_at_coordinate == [(2, 2), (2, 3), (3, 2), (3, 3)]
-
-        pixels_at_coordinate = aa.util.grid.positions_at_coordinate_from(
-            grid_2d=grid.in_2d, coordinate=(0.6, 0.6)
-        )
-
-        assert pixels_at_coordinate == [(1, 2), (1, 3), (2, 2), (2, 3)]
-
-        pixels_at_coordinate = aa.util.grid.positions_at_coordinate_from(
-            grid_2d=grid.in_2d, coordinate=(1.1, 1.1)
-        )
-
-        assert pixels_at_coordinate == [(1, 3)]
-
-    def test__uniform_grid__mask_remove_points(self):
-
-        grid = aa.Grid.uniform(shape_2d=(5, 5), pixel_scales=1.0)
-
-        mask = aa.Mask.manual(
-            mask=[
-                [True, True, False, False, False],
-                [True, True, False, False, False],
-                [True, True, False, False, False],
-                [True, True, True, True, True],
-                [True, True, True, True, True],
+        grid_1d = np.array(
+            [
+                [1.0, -1.0],
+                [1.0, 0.0],
+                [1.0, 1.0],
+                [0.0, -1.0],
+                [0.0, 0.0],
+                [0.0, 1.0],
+                [-1.0, -1.0],
+                [-1.0, 0.0],
+                [-1.0, 1.0],
             ]
         )
 
-        pixels_at_coordinate = aa.util.grid.positions_at_coordinate_from(
-            grid_2d=grid.in_2d, coordinate=(0.3, 0.3), mask=mask
+        grid_pixel_centres_1d, y_shape, x_shape = aa.util.grid.grid_pixel_centres_1d_via_grid_1d_overlap(
+            grid_1d=grid_1d, pixel_scales=(1.0, 1.0), buffer=1
         )
 
-        assert pixels_at_coordinate == [(1, 3)]
+        assert (
+            grid_pixel_centres_1d
+            == np.array(
+                [[1, 1], [1, 2], [1, 3], [2, 1], [2, 2], [2, 3], [3, 1], [3, 2], [3, 3]]
+            )
+        ).all()
+        assert (y_shape, x_shape) == (5, 5)
 
-    def test__non_uniform_grid__locates_multiple_pixels_correctly(self):
+        grid_1d = np.array(
+            [
+                [3.0, 1.0],
+                [3.0, 2.0],
+                [3.0, 3.0],
+                [2.0, 1.0],
+                [2.0, 2.0],
+                [2.0, 3.0],
+                [1.0, 1.0],
+                [1.0, 2.0],
+                [1.0, 3.0],
+            ]
+        )
 
-        grid = aa.Grid.manual_2d(
-            grid=[
+        grid_pixel_centres_1d, y_shape, x_shape = aa.util.grid.grid_pixel_centres_1d_via_grid_1d_overlap(
+            grid_1d=grid_1d, pixel_scales=(1.0, 1.0), buffer=1
+        )
+
+        assert (
+            grid_pixel_centres_1d
+            == np.array(
+                [[1, 1], [1, 2], [1, 3], [2, 1], [2, 2], [2, 3], [3, 1], [3, 2], [3, 3]]
+            )
+        ).all()
+        assert (y_shape, x_shape) == (5, 5)
+
+        grid_1d = np.array([[3.0, 3.0], [3.0, 1.0], [0.0, 3.0], [2.0, 2.0]])
+
+        grid_pixel_centres_1d, y_shape, x_shape = aa.util.grid.grid_pixel_centres_1d_via_grid_1d_overlap(
+            grid_1d=grid_1d, pixel_scales=(1.0, 1.0), buffer=1
+        )
+
+        assert (
+            grid_pixel_centres_1d == np.array([[1, 3], [1, 1], [4, 3], [2, 2]])
+        ).all()
+        assert (y_shape, x_shape) == (6, 5)
+
+
+class TestGridBuffedFrom:
+    def test__simple_grids_buffed_correctly(self):
+
+        grid_1d = np.array([[0.0, 0.0]])
+
+        grid_buffed_1d, y_shape, x_shape = aa.util.grid.grid_buffed_from(
+            grid_1d=grid_1d, pixel_scales=(1.0, 1.0), buffer=1
+        )
+
+        assert (
+            grid_buffed_1d
+            == np.array(
+                [
+                    [1.0, -1.0],
+                    [1.0, 0.0],
+                    [1.0, 1.0],
+                    [0.0, -1.0],
+                    [0.0, 0.0],
+                    [0.0, 1.0],
+                    [-1.0, -1.0],
+                    [-1.0, 0.0],
+                    [-1.0, 1.0],
+                ]
+            )
+        ).all()
+
+        grid_1d = np.array([[1.0, 1.0]])
+
+        grid_buffed_1d, y_shape, x_shape = aa.util.grid.grid_buffed_from(
+            grid_1d=grid_1d, pixel_scales=(1.0, 1.0), buffer=1
+        )
+
+        assert (
+            grid_buffed_1d
+            == np.array(
+                [
+                    [2.0, 0.0],
+                    [2.0, 1.0],
+                    [2.0, 2.0],
+                    [1.0, 0.0],
+                    [1.0, 1.0],
+                    [1.0, 2.0],
+                    [0.0, 0.0],
+                    [0.0, 1.0],
+                    [0.0, 2.0],
+                ]
+            )
+        ).all()
+
+        grid_1d = np.array([[1.0, 1.0], [2.0, 2.0]])
+
+        grid_buffed_1d, y_shape, x_shape = aa.util.grid.grid_buffed_from(
+            grid_1d=grid_1d, pixel_scales=(1.0, 1.0), buffer=1
+        )
+
+        assert (
+            grid_buffed_1d
+            == np.array(
                 [
                     [3.0, 1.0],
-                    [0.0, 0.0],
+                    [3.0, 2.0],
                     [3.0, 3.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                ],
-                [
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                ],
-                [
+                    [2.0, 0.0],
+                    [2.0, 1.0],
+                    [2.0, 2.0],
+                    [2.0, 3.0],
+                    [1.0, 0.0],
                     [1.0, 1.0],
-                    [0.0, 0.0],
+                    [1.0, 2.0],
                     [1.0, 3.0],
                     [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                ],
+                    [0.0, 1.0],
+                    [0.0, 2.0],
+                ]
+            )
+        ).all()
+
+        grid_1d = np.array([[1.0, 1.0], [3.0, 3.0]])
+
+        grid_buffed_1d, y_shape, x_shape = aa.util.grid.grid_buffed_from(
+            grid_1d=grid_1d, pixel_scales=(2.0, 2.0), buffer=1
+        )
+
+        assert (
+            grid_buffed_1d
+            == np.array(
                 [
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [1.0, 3.0],
-                    [0.0, 0.0],
-                    [1.0, 1.0],
-                ],
-                [
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                ],
-                [
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [3.0, 3.0],
-                    [0.0, 0.0],
+                    [5.0, 1.0],
+                    [5.0, 3.0],
+                    [5.0, 5.0],
+                    [3.0, -1.0],
                     [3.0, 1.0],
-                ],
+                    [3.0, 3.0],
+                    [3.0, 5.0],
+                    [1.0, -1.0],
+                    [1.0, 1.0],
+                    [1.0, 3.0],
+                    [1.0, 5.0],
+                    [-1.0, -1.0],
+                    [-1.0, 1.0],
+                    [-1.0, 3.0],
+                ]
+            )
+        ).all()
+
+
+class TestGridUpscaled1d:
+    def test__simple_grid_in_output_grid_is_upscaled(self):
+
+        grid_1d = np.array([[1.0, 1.0]])
+
+        grid_upscaled_1d = aa.util.grid.grid_upscaled_1d_from(
+            grid_1d=grid_1d, upscale_factor=1, pixel_scales=(2.0, 2.0)
+        )
+
+        assert (grid_upscaled_1d == np.array([[1.0, 1.0]])).all()
+
+        grid_upscaled_1d = aa.util.grid.grid_upscaled_1d_from(
+            grid_1d=grid_1d, upscale_factor=2, pixel_scales=(2.0, 2.0)
+        )
+
+        assert (
+            grid_upscaled_1d
+            == np.array([[1.5, 0.5], [1.5, 1.5], [0.5, 0.5], [0.5, 1.5]])
+        ).all()
+
+        grid_1d = np.array([[1.0, 1.0], [1.0, 3.0]])
+
+        grid_upscaled_1d = aa.util.grid.grid_upscaled_1d_from(
+            grid_1d=grid_1d, upscale_factor=2, pixel_scales=(2.0, 2.0)
+        )
+
+        assert (
+            grid_upscaled_1d
+            == np.array(
                 [
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                    [0.0, 0.0],
-                ],
-            ],
-            pixel_scales=1.0,
+                    [1.5, 0.5],
+                    [1.5, 1.5],
+                    [0.5, 0.5],
+                    [0.5, 1.5],
+                    [1.5, 2.5],
+                    [1.5, 3.5],
+                    [0.5, 2.5],
+                    [0.5, 3.5],
+                ]
+            )
+        ).all()
+
+        grid_1d = np.array([[1.0, 1.0], [3.0, 1.0]])
+
+        grid_upscaled_1d = aa.util.grid.grid_upscaled_1d_from(
+            grid_1d=grid_1d, upscale_factor=2, pixel_scales=(2.0, 2.0)
         )
 
-        pixels_at_coordinate = aa.util.grid.positions_at_coordinate_from(
-            grid_2d=grid.in_2d, coordinate=(2.0, 2.0)
+        assert (
+            grid_upscaled_1d
+            == np.array(
+                [
+                    [1.5, 0.5],
+                    [1.5, 1.5],
+                    [0.5, 0.5],
+                    [0.5, 1.5],
+                    [3.5, 0.5],
+                    [3.5, 1.5],
+                    [2.5, 0.5],
+                    [2.5, 1.5],
+                ]
+            )
+        ).all()
+
+        grid_1d = np.array([[1.0, 1.0]])
+
+        grid_upscaled_1d = aa.util.grid.grid_upscaled_1d_from(
+            grid_1d=grid_1d, upscale_factor=2, pixel_scales=(3.0, 2.0)
         )
 
-        assert pixels_at_coordinate == [(1, 1), (4, 5)]
+        assert (
+            grid_upscaled_1d
+            == np.array([[1.75, 0.5], [1.75, 1.5], [0.25, 0.5], [0.25, 1.5]])
+        ).all()
+
+        grid_upscaled_1d = aa.util.grid.grid_upscaled_1d_from(
+            grid_1d=grid_1d, upscale_factor=3, pixel_scales=(2.0, 2.0)
+        )
+
+        assert grid_upscaled_1d[0] == pytest.approx(np.array([1.666, 0.333]), 1.0e-2)
+        assert grid_upscaled_1d[1] == pytest.approx(np.array([1.666, 1.0]), 1.0e-2)
+        assert grid_upscaled_1d[2] == pytest.approx(np.array([1.666, 1.666]), 1.0e-2)
+        assert grid_upscaled_1d[3] == pytest.approx(np.array([1.0, 0.333]), 1.0e-2)
+        assert grid_upscaled_1d[4] == pytest.approx(np.array([1.0, 1.0]), 1.0e-2)
+        assert grid_upscaled_1d[5] == pytest.approx(np.array([1.0, 1.666]), 1.0e-2)
+        assert grid_upscaled_1d[6] == pytest.approx(np.array([0.333, 0.333]), 1.0e-2)
+        assert grid_upscaled_1d[7] == pytest.approx(np.array([0.333, 1.0]), 1.0e-2)
+        assert grid_upscaled_1d[8] == pytest.approx(np.array([0.333, 1.666]), 1.0e-2)
